/* bidirectional search algorithms library
    Copyright (C) 2010 Thomas Schnattinger

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see http://www.gnu.org/licenses/ .
*/
/*! \file createindex.cpp
    \brief This program creates indexes
	\author Thomas Schnattinger
*/
#include <cstdio>
#include <iostream>
#include <sdsl/testutils.hpp>
#include <sdsl/coder.hpp>
#include "indexes.hpp"

using namespace std;
using namespace sdsl;

template<class Index>
void create_index(unsigned char *text, char* filename) {
	Index index(text);
	ofstream out(filename);
	index.serialize(out);
	out.close();
}

int main(int argc, char** argv) {

	if (argc != 4) {
		printf("\n");
		printf("Creates an index from the given file\n");
		printf("\n");
		printf("usage: %s inputfile #index(1..6) indexfile\n\n", argv[0]);
		printf(" # | Index\n");
		printf("---+-----------------------------------\n");
		printf(" 1 | index_sa_text_psi\n");
		printf(" 2 | index_sa_isa\n");
		printf(" 3 | index_sa_text_occ<>\n");
		printf(" 4 | index_csa_psi_text<>\n");
		printf(" 5 | index_csa_fmi_text<>\n");
		printf(" 6 | index_bidirectional_waveletindex<>\n");
		printf("\n");
		return 1;
	}

	char* inputfile = argv[1];
	int index = atoi(argv[2]);
	char* indexfile = argv[3];

	unsigned char *text = NULL;
	if( file::readText(inputfile, (char* &)text) ) {
		switch (index) {
			case 1:
				create_index<index_sa_text_psi>(text, indexfile);
				break;
			case 2:
				create_index<index_sa_isa>(text, indexfile);
				break;
			case 3:
				create_index<index_sa_text_occ<> >(text, indexfile);
				break;
			case 4:
				create_index<index_csa_psi_text<> >(text, indexfile);
				break;
			case 5:
				create_index<index_csa_fmi_text<> >(text, indexfile);
				break;
			case 6:
				create_index<index_bidirectional_waveletindex<> >(text, indexfile);
				break;
		}
		delete [] text;
	}

	return 0;
}
