/* bidirectional search algorithms library
    Copyright (C) 2010 Thomas Schnattinger

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see http://www.gnu.org/licenses/ .
*/
/*! \file test_indexes.cpp
    \brief Demo program
	\author Thomas Schnattinger
*/
#include <cstdio>
#include "indexes.hpp"
#include <sdsl/testutils.hpp>

using namespace std;

template <class Index>
void test_index_forward() {
	typedef typename Index::searchstate searchstate;

	const unsigned char* text = (const unsigned char*) "el_anele_lepanelen";
	Index index(text);

	printf("Test Forward Search...\n");
	printf("Index size: %d\n", index.used_bytes());

	searchstate s;
	index.init_search_state(s);

	printf("[%d..%d]\n", s.occ_begin+1, s.occ_end);
	index.forward_search(index.char2comp['e'], s);
	printf("[%d..%d]\n", s.occ_begin+1, s.occ_end);
	index.forward_search(index.char2comp['l'], s);
	printf("[%d..%d]\n", s.occ_begin+1, s.occ_end);
	index.forward_search(index.char2comp['e'], s);
	printf("[%d..%d]\n", s.occ_begin+1, s.occ_end);
	int_vector<> occ;
	index.extract_sa(s.occ_begin, s.occ_end, occ);
	for (size_t i=0; i<occ.size(); i++) {
		printf("  SA[%llu] = %llu\n", s.occ_begin+1+i, (uint64_t)occ[i]);
	}
}

template <class Index>
void test_index_backward() {
	typedef typename Index::searchstate searchstate;

	const unsigned char* text = (const unsigned char*) "el_anele_lepanelen";
	Index index(text);

	printf("Test Backward Search...\n");
	printf("Index size: %d\n", index.used_bytes());

	searchstate s;
	index.init_search_state(s);

	printf("[%d..%d]\n", s.occ_begin+1, s.occ_end);
	index.backward_search(index.char2comp['e'], s);
	printf("[%d..%d]\n", s.occ_begin+1, s.occ_end);
	index.backward_search(index.char2comp['l'], s);
	printf("[%d..%d]\n", s.occ_begin+1, s.occ_end);
	index.backward_search(index.char2comp['e'], s);
	printf("[%d..%d]\n", s.occ_begin+1, s.occ_end);
	int_vector<> occ;
	index.extract_sa(s.occ_begin, s.occ_end, occ);
	for (size_t i=0; i<occ.size(); i++) {
		printf("  SA[%llu] = %llu\n", s.occ_begin+1+i, (uint64_t)occ[i]);
	}
}

int main() {

	printf("\nTeste index_sa_text_psi:\n");
	test_index_forward<index_sa_text_psi >();
	test_index_backward<index_sa_text_psi >();

	printf("\nTeste index_sa_isa:\n");
	test_index_forward<index_sa_isa>();
	test_index_backward<index_sa_isa>();

	printf("\nTeste index_sa_text_occ:\n");
	test_index_forward<index_sa_text_occ<> >();
	test_index_backward<index_sa_text_occ<> >();

	printf("\nTeste index_csa_psi_text:\n");
	test_index_forward<index_csa_psi_text<> >();
	test_index_backward<index_csa_psi_text<> >();

	printf("\nTeste index_csa_fmi_text:\n");
	test_index_forward<index_csa_fmi_text<> >();
	test_index_backward<index_csa_fmi_text<> >();

	printf("\nTeste index_bidirectional_waveletindex:\n");
	test_index_forward<index_bidirectional_waveletindex<> >();
	test_index_backward<index_bidirectional_waveletindex<> >();

	return 0;
}
