#include <sdsl/vectors.hpp>
#include <sdsl/algorithms.hpp>
#include <sdsl/testutils.hpp>
#include <sdsl/suffixarrays.hpp>
#include <string>
#include <cstdlib>

using namespace sdsl;
using std::string;

typedef csa_sada_prac<enc_vector_prac<coder::fibonacci, 90>, 20> tCSAFibP;

int main(int argc, char* argv[]){
	string path;
	if(argc<2){
		return EXIT_FAILURE;
	}
	path = argv[1];	
	char *c = NULL; 
	uint64_t size = file::readText(path.c_str(), c);
	if( c!=NULL )
	{
	
		tCSAFibP csa;
		if( util::load_from_file(csa, (path+".CSA").c_str()) ){
			std::cout<<"csa loaded. please enter patterns or \"quit.\" to terminate"<<std::endl;
			string pattern;
			tCSAFibP::size_type begin, end;
			while(std::cin>>pattern and pattern.size()>0 and pattern!="quit."){
				algorithm::backward_search(csa, (const unsigned char*)pattern.c_str(), pattern.size(), begin, end);
				std::cout<<"pattern \""<<pattern<<"\" occures "<<end-begin<<" times"<<std::endl;
				if( end-begin > 0){
					tCSAFibP::size_type suffix;
					uint32_t occ = 1;
					string next;
					do{
						suffix = csa[begin+occ-1];
						const uint32_t lines = 5;
						uint32_t pre = lines*82, post = lines*82;
						if( suffix < pre )
							pre = suffix;
						if( suffix+post >= csa.size() )
							post = csa.size()-suffix;
						string context = algorithm::extract(csa, suffix-pre, suffix+post);
						std::cout<<"context of occurence "<<occ<<" of "<<end-begin<<": \n"<< context <<"\n"<<std::endl;
						++occ;
					}
					while(begin+occ<end and std::cin>>next and next=="n");
				}
			}
		}
		else{
			int_vector<> sa(size, 0, bit_magic::l1BP(size)+1 );
			if( util::load_from_file(sa, (path+".SA").c_str())  ){
				std::cout<<"SA already calculated."<<std::endl;
				tCSAFibP csat(csa_uncompressed(sa, (const unsigned char*)c), (unsigned char*) c);
				std::cout<<"CSA created."<<std::endl;
				util::store_to_file(csat, (path+".CSA").c_str());
			}
			else{
				std::cout<<"creating suffix array for file "<<path<<" of "<<size<<" bytes"<<std::endl;
				algorithm::calculate_sa((unsigned char*)c, size,  sa);
				util::store_to_file(sa, (path+".SA").c_str());
			}
		}
		delete c;
	}
	else{
		std::cout<<"Could not open file "<<path<<std::endl;
	}
}
