#include "sdsl/int_vector.hpp"
#include "sdsl/util.hpp"
#include "sdsl/testutils.hpp"
#include "sdsl/algorithms.hpp"
#include "sdsl/bitmagic.hpp"
#include "sdsl/wavelet_tree.hpp"
#include "sdsl/rank_support.hpp"
#include "sdsl/balanced_parentheses_support.hpp"
#include "sdsl/lcp.hpp"
#include <iostream>
#include <string>
#include <algorithm>
#include <iomanip>

using namespace sdsl;
using namespace std;
	 
int main(){
	typedef int_vector<>::size_type size_type;
	string sa_tmp_file = "/tmp/" + util::to_string(util::get_pid()) + ".sa";
	string bwt_tmp_file = "/tmp/" + util::to_string(util::get_pid()) + ".bwt";
	string test_file = "../performance/test_cases/english.50MB";// "dna_pure.50MB";

	stop_watch sw;

	char *c_str = NULL;
	size_type n=0;
	if( (n=file::readText(test_file.c_str(), c_str)) == 0){
		cout<<"Could not read file \""<<test_file<<"\"! Please download it from the pizza and chili website and place it in the current directory to run the tests"<<endl;
		return 0;
	}
	cout<<"n="<<n<<endl;
	int_vector<> sa(n+1, 0, std::max( (size_type)bit_magic::l1BP(n+1)+1, (size_type)8) );
	sw.start();
	algorithm::calculate_sa((const unsigned char*)c_str, n+1, sa);
	sw.stop();
	
	cout<<"calculate sa takes "<<sw.getUserTime()<<" ms."<<endl;

	sw.start();
	util::store_to_file(sa, sa_tmp_file.c_str() );
	sw.stop();
	cout<<"saving sa to disk takes "<<sw.getUserTime()<<" ms."<<endl;
/*
	sw.start();
	// calculate bwt and use sa to store it in main memory
	for(size_type i=0; i<sa.size(); ++i){
		//sa.setInt( i<<3, c_str[(sa[i]+sa.size()-1)%sa.size()], 8);
		sa[i] = c_str[(sa[i]+sa.size()-1)%sa.size()];
	}
//	sa.setIntWidth(8);
//	sa.resize(sa.size());
	sw.stop();
	cout<<"calculate bwt takes "<<sw.getUserTime()<<" ms."<<endl;



	for(size_type i=0; i<sa.size(); ++i)
		c_str[i] = sa[i];
	sa.resize(0);

*/	
/*
	sw.start();
	wavelet_tree<> wt((unsigned char*)c_str, n+1);
	sw.stop();
	cout<<"calculate wavelet tree takes "<<sw.getUserTime()<<" ms."<<endl;
*/
	delete [] c_str;
	c_str = NULL;
	if( (n=file::readText(test_file.c_str(), c_str)) == 0){
		cout<<"Could not read file \""<<test_file<<"\"! Please download it from the pizza and chili website and place it in the current directory to run the tests"<<endl;
		return 0;
	}
	util::load_from_file(sa, sa_tmp_file.c_str());

	size_type C[256] = {0};
	for(size_type i=0; i<256; ++i) C[i] = 0;
	for(size_type i=0; i<sa.size(); ++i){
		C[(unsigned char)c_str[i]]++;
	}
	for(size_type i=256; i!=0; --i){
		C[i] = C[i-1];
	}
	C[0] = 0;
	for(size_type i=1; i<256; ++i){
		C[i] += C[i-1];
	}

	sw.start();
	size_type l=0, sa_1=0;
	for(size_type i=0; i<sa.size(); ++i){
		if(sa[i]==0){
			sa_1 = i; break;
		}
//		cout<<setw(3)<<i<<" "<<setw(3)<<sa[i]<<endl;
	}
	int_vector<> isa = sa;
//	for(size_type i=0; i<sa.size(); ++i){
//		isa[sa[i]] = i;
//	}
	// TODO: check function!
	algorithm::inverse_permutation_inplace(isa);

	for(size_type i=0; i<sa.size(); ++i){
		if(sa[isa[i]]!=i){
			cout<<"i="<<i<<" sa[isa[i]]]="<<sa[isa[i]]<<" isa[i]="<<isa[i]<<endl;
			for(size_type j=0; j<sa.size();++j){
				if(sa[j]==i)
					cout<<"isa[i]="<<j<<endl;
			}
			cout<<"inverse permutation error"<<endl;
			return -1;
		}
	}

	cout<<"calculated C"<<endl;
	for(size_type i=0, j=0; i<sa.size(); ++i){
		sa_1 = isa[i];	
		if( sa_1 == 0 ){
			sa[sa.size()-1] = 0;
		}else{
			j = sa[sa_1-1];
			while( i+l < sa.size() and j+l < sa.size() and c_str[i+l]==c_str[j+l] )	
				++l;
			sa[sa_1-1] = l;
		} 
		//cout<<"i="<<setw(10)<<i<<" sa_1[i]="<<setw(10)<<sa_1<<" c_str[i]="<<c_str[i]<<" ("<<setw(3)<<(unsigned int)c_str[i]<<") C[c_str[i]]="<<setw(10)<<C[(unsigned char)c_str[i]]<<"l="<<l<<endl;
		if( l > 0 )
			--l;
		//sa_1 = wt.select( sa_1 - C[(unsigned char)c_str[i]] + 1, c_str[i] );
	}

	size_type lcp_val = sa[sa.size()-1];
	for(size_type i=0, t; i<sa.size(); ++i){
		t = sa[i];
		sa[i] = lcp_val;
		lcp_val = t;
//		cout<<setw(3)<<i<<" "<<setw(3)<<sa[i]<<endl;
	}

	sw.stop();
	cout<<"calculate lcp array takes "<<sw.getUserTime()<<" ms."<<endl;

	delete [] c_str;
	c_str = NULL;

	bit_vector sct_bp;
	sw.start();
	algorithm::construct_supercartesian_tree_bp_succinct(sa, sct_bp);
	sw.stop();
	cout<<"calculate bp of supercartesian tree takes "<<sw.getUserTime()<<" ms."<<endl;

	for(size_type i=0; i< std::min((size_type)20, sct_bp.size()); ++i)
		if(sct_bp[i])
			cout<<"(";
		else
			cout<<")";
	cout<<endl;

	rank_support_v<10, 2> rank10_sct_bp(&sct_bp);
	cout<<"leaves in sct_bp "<<rank10_sct_bp.rank(sct_bp.size())<<endl;
	cout<<"create balanced parentheses support"<<endl;
	balanced_parentheses_support_simple<> sct_bp_support(&sct_bp);

	bit_vector first_p_idx;
	sw.start();
	algorithm::calculate_first_p_index(sa, first_p_idx);
	sw.stop();
	cout<<"calculate bp of first_p_idx takes "<<sw.getUserTime()<<" ms."<<endl;
	for(size_type i=0; i< std::min((size_type)20, first_p_idx.size()); ++i)
		cout<<first_p_idx[i];
	cout<<endl;

	rank_support_v<> first_p_idx_rnk(&first_p_idx);

	size_type first_p_ones =  first_p_idx_rnk.rank(first_p_idx.size());
	cout<<"cnt_frst_p = "<< first_p_idx_rnk.rank(first_p_idx.size()) <<"/"<<first_p_idx.size()<<endl;

	int_vector<32> lcp(first_p_idx_rnk.rank(first_p_idx.size()), 0);	
	size_type cnt_reducible=0, big_values=0, cnt_red_and_big=0, max_lcp=0, in_range_cnt=0, red_in_range_cnt=0;
	double avg_lcp=0;
	for(size_type i=0, k=0, close=0; i< sct_bp.size(); ++i)
		if( sct_bp[i]==0 and first_p_idx[close++]==1 ){
//			size_type foi = sct_bp_support.find_open(i);
//			if(k<20){
//				cout<<"i="<<i<<" foi[i]="<<foi<<endl;
//				cout<<"sct_bp_support.rank(foi)="<<sct_bp_support.rank(foi)<<endl;
//			}
			lcp[k++] = sa[ sct_bp_support.rank(sct_bp_support.find_open(i))-1 ];
			if(k<800)
				cout<<lcp[k-1]<<" ";

			if(k+800> first_p_ones )
				cout<<lcp[k-1]<<",";
			if(lcp[k-1]>=255)
				++big_values;
			if(lcp[k-1]>max_lcp)
				max_lcp=lcp[k-1];
			avg_lcp += lcp[k-1];
			
			if( i>1 and sct_bp[i-2]==0 and first_p_idx[close-2]==1){
				if( lcp[k-1] < lcp[k-2] ){
					cnt_reducible++;
					if(lcp[k-1]>=255)
						cnt_red_and_big++;
					if( lcp[k-1]+255 >= lcp[k-2] ){
						red_in_range_cnt++;
					}
				}
				else
					cout<<" i = "<<i<<" komisch"<<endl;
			}
			if(k>1){
				if( lcp[k-1] <= lcp[k-2]+128 and lcp[k-1]+128 >= lcp[k-2] )
					in_range_cnt++;
			}

//			if( (k%100000)==0 )
//				cout<<"k="<<k<<endl;
		}
	cout<<endl;
	if(lcp.size())
		avg_lcp/=lcp.size();
	cout<<"calculated lcp_new"<<endl;
	cout<<"ratio of big values = "<<big_values<<" / "<<lcp.size()<<endl;
	cout<<"reducible values = "<<cnt_reducible<<" / "<<lcp.size()<<endl;
	cout<<"ratio of big_values of reducibale values = "<<cnt_red_and_big<<" / "<<cnt_reducible<<endl;
	cout<<"ratio of in_range values = "<<in_range_cnt<<" / "<<lcp.size()<<endl;
	cout<<"ratio of red_in_range values = "<<red_in_range_cnt<<" / "<<lcp.size()<<endl;
	cout<<"max lcp value = "<<max_lcp<<endl;
	cout<<"avg lcp="<<avg_lcp<<endl;
	

	sw.start();
	for(size_type i=0; i<sa.size(); ++i){
		size_type r = sct_bp_support.select(i+1);	
		r = sct_bp_support.find_close(r);
		r = (r+1)-sct_bp_support.rank(r);
		r = first_p_idx_rnk.rank(r) - (first_p_idx[r-1]);
		r = lcp[r];
		if( r != sa[i]){
			std::cout<<"i="<<i<<" r="<<r<<" lcp[i]="<<sa[i]<<std::endl;
			break;
		}
	}
	sw.stop();
	cout<<"comparison of lcp array took "<<sw.getUserTime()<<" ms"<<endl;

	lcp_kurtz<int_vector<> > lcp1b(lcp); 

	sw.start();
	for(size_type i=0; i<sa.size(); ++i){
		size_type r = sct_bp_support.select(i+1);	
		r = sct_bp_support.find_close(r);
		r = (r+1)-sct_bp_support.rank(r);
		r = first_p_idx_rnk.rank(r) - (first_p_idx[r-1]);
		r = lcp1b[r];
		if( r != sa[i]){
			std::cout<<"i="<<i<<" r="<<r<<" lcp[i]="<<sa[i]<<std::endl;
			break;
		}
	}
	sw.stop();
	cout<<"comparison of lcp1b array took "<<sw.getUserTime()<<" ms"<<endl;

	cout<< (util::get_size_in_bytes(lcp1b)+util::get_size_in_bytes(first_p_idx)+util::get_size_in_bytes(first_p_idx_rnk))/((double)sa.size()) <<endl;
	cout<< (util::get_size_in_bytes(lcp1b))/((double)sa.size()) <<endl;
	cout<< (util::get_size_in_bytes(first_p_idx))/((double)sa.size()) <<endl;
	cout<< (util::get_size_in_bytes(first_p_idx_rnk))/((double)sa.size()) <<endl;
	cout<< (util::get_size_in_bytes(sct_bp))/((double)sa.size()) <<endl;
	cout<< (util::get_size_in_bytes(sct_bp_support))/((double)sa.size()) <<endl;


//	std::cerr<<"lcp size = "<< util::get_size_in_bytes() << std::endl;


	std::remove(sa_tmp_file.c_str());
}
