#include "sdsl/suffixtrees.hpp" // include the header for suffix trees 
#include <algorithm>

using namespace sdsl; // use the sdsl namespace

typedef cst_sct<> tST; // typedef for the used suffix tree

int main(){
	// We take two string s1 and s2 and construct s=s1#s2 
	// (where # is a special symbol which does neither occure in s1 nor in s2).
	unsigned char s[64] 	= "brautkleidbleibtbrautkleid#undblaukrautbleibtblaukraut";
	tST::size_type sep_pos  = 26;
	// no we build the compressed suffix tree of s
	tST cst(s);
	// to solve the maximal unique matches (MUM) problem we 
	// search inner nodes v which 
	//   - have exactly two leaves 
	// 	 - one leaf belongs to s1 and one to s2
	//   - the characters before the two suffixes are not equal

	for(tST::iterator v=cst.begin(); v!=cst.end(); ++v){
		if( !cst.is_leaf(*v) ){ // *v is a inner node
			if(v.visit() == 1){ // visited the first time in the dfs
				if( cst.children(*v) == 2 ){
					tST::node_type c1	= cst.ith_child(*v, 1);
				   	tST::node_type c2	= cst.ith_child(*v, 2);
					if( cst.is_leaf(c1) and cst.is_leaf(c2)  ){
						tST::size_type sn1 = cst.sn(c1), sn2 = cst.sn(c2);
						if( (sn1 < sep_pos and sn2 > sep_pos) or (sn2 < sep_pos and sn1 > sep_pos) ){
							if( sn1 == 0 or sn2 ==0 or s[sn1-1]!=s[sn2-1]  ){
								std::swap( s[63], s[sn1+cst.depth(*v)] );
								std::cout<<"found MUM \""<< (s+sn1) <<"\""<<std::endl;
								std::swap( s[63], s[sn1+cst.depth(*v)] );
							}	
						}
					}
				}
			}
		}
	}
}
