#include <iostream>
#include <sdsl/int_vector.hpp>
#include <sdsl/rank_support_v.hpp>
#include <sdsl/rank_support_v5.hpp>
#include <sdsl/util.hpp>
#include <sdsl/testutils.hpp>

using namespace std;
using namespace sdsl;

template<class RS>
void run_rank_support(const RS &rs, const bit_vector &b){
	typedef typename RS::size_type size_type;
	size_type k=0;
	stop_watch sw;
	sw.start();
	for(size_type i=0, x=123; i<b.size(); ++i){
		k += rs(x);
		x = (x*x-1)%b.size();
	}
	sw.stop();

	cout << util::demangle(typeid(RS).name()) << " for "<<b.size()<<" elements took "<<sw.getUserTime()<<" ms k="<< k <<endl;
	
}

int main(){
	typedef bit_vector::size_type size_type;
	srand(time(0));
	bit_vector b(10000000+(rand()%2048));
	util::set_random_bits(b);
	rank_support_v<>  rs(&b);
	rank_support_v5<> rs5(&b);
	size_type falsch=0;
	for(size_type i=0; i<b.size(); ++i){
		size_type r1,r2;
		r1 = rs(i);
		r2 = rs5(i);
		if( r1!=r2 ){
			cout<<"i="<<i<<" rs(i)="<<r1<<" rs5(i)="<<r2<<endl;
			if( ++falsch > 10 )
				break;
		}
	}
	run_rank_support(rs, b);
	run_rank_support(rs5, b);
}
