#include <sdsl/sorted_stack_support.hpp>
#include <sdsl/testutils.hpp>
#include <stack>
#include <algorithm>
#include <iostream>
#include <vector>

using namespace std;
using namespace sdsl;

typedef vector<uint32_t> tVI;
typedef stack<uint32_t> tSI;

int main(){
	tVI a(100000000);
	for(size_t i=0; i<a.size(); ++i)
	{
		a[i] = rand()%1000000; // the std::stack is 2-3 times faster
//		a[i] = i;              // both methods run at equal speed
	}
	tSI s1;
	sorted_stack_support<tVI> s2(&a);
	for(size_t i=0; i<a.size(); ++i)
	{
		while( !s1.empty() and s1.top() >= a[i] ){
			s1.pop();
			s2.pop();
		}
		s1.push(a[i]);
		s2.push(i);
		if( s1.top() != a[s2.top()] ){
			cout<<s1.top()<< " "<<a[s2.top()]<<" "<<s1.size()<<" "<<s2.size()<<endl;
		}
	}
	while( !s1.empty() ) s1.pop();
	while( !s2.empty() ) s2.pop();
	std::cout<<"checked equal results"<<std::endl;

	size_t k=0;

	stop_watch stopwatch;
	stopwatch.start();
	for(size_t i=0; i<a.size(); ++i)
	{
		while( !s1.empty() and s1.top() >= a[i] ){
			k+=s1.top();
			s1.pop();
		}
		s1.push(a[i]);
	}
	stopwatch.stop();
	while( !s1.empty() ) s1.pop();
	cout<<"Time for std::stack "<<stopwatch.getUserTime()<<endl;
	cout<<"checksum "<<k<<endl;

	k = 0;
	stopwatch.start();
	for(size_t i=0; i<a.size(); ++i)
	{
		while( !s2.empty() and a[s2.top()] >= a[i] ){
			k+=a[s2.top()];
			s2.pop();
		}
		s2.push(i);
	}
	stopwatch.stop();
	while( !s2.empty() ) s2.pop();
	cout<<"Time for sorted_int_stack "<<stopwatch.getUserTime()<<endl;
	cout<<"checksum "<<k<<endl;
}
