#include "sdsl/suffixtrees.hpp"
#include "sdsl/csa_uncompressed.hpp"
#include "sdsl/csa_wt.hpp"
#include "sdsl/int_vector.hpp"
#include "sdsl/util.hpp"
#include "sdsl/testutils.hpp"
#include "sdsl/algorithms.hpp"
#include "sdsl/lcp_kurtz.hpp"
#include "sdsl/lcp_uncompressed.hpp"
#include <iostream>
#include <string>
#include <algorithm>

using namespace sdsl;
using namespace std;

//typedef cst_sct<> 	full_compressed_cst_sct;
typedef cst_sct<csa_wt<>  > 	full_compressed_cst_sct;

typedef cst_sada<>	full_compressed_cst_sada;
typedef cst_sct<cst_sct<>::csa_type, lcp_uncompressed<> > lcp_uncompressed_cst_sct;
typedef cst_sada<cst_sada<>::csa_type, lcp_uncompressed<> > lcp_uncompressed_cst_sada;
typedef cst_sct<csa_uncompressed, lcp_uncompressed<> > full_uncompressed_cst_sct;
typedef cst_sada<csa_uncompressed, lcp_uncompressed<> > full_uncompressed_cst_sada;

typedef cst_sct<cst_sct<>::csa_type, lcp_kurtz<cst_sct<>::csa_type > > 	full_compressed_cst_sct_kurtz;

template<class CST>
void print_tree(const CST& cst){
	for(typename CST::iterator i=cst.begin(); i!=cst.end(); ++i){
		if( i.visit() == 1){
			cout<<algorithm::extract(cst, *i)<<endl;
			for(typename CST::size_type j=0;j<cst.depth(*i); ++j)
				cout<<" ";
			if( !cst.is_leaf(*i) ){
				cout<<*i<<endl;
			}
			else{
					cout<<" "<<*i<<" sa-number="<<cst.sn(*i)<<endl;
			}
		}
	}
}

template<class CST>
typename CST::size_type dfs_traverse(const CST& cst){
	stop_watch sw;
	sw.start();
	typename CST::size_type sum_depth = 0;
	for(typename CST::iterator i=cst.begin(); i!=cst.end(); ++i){
		sum_depth += cst.depth(*i);
	}
	sw.stop();
	cout << sw.getUserTime() << " ms /  sum = "<< sum_depth << endl;
	return sum_depth;
}

template<class CST>
typename CST::size_type dfs_traverse_recursive(const CST& cst){
	stop_watch sw;
	sw.start();
	typename CST::size_type sum_depth = dfs_traverse_rec(cst, cst.root());
	sw.stop();
	cout << sw.getUserTime() << " ms /  sum = "<< sum_depth << endl;
	return sum_depth;

}

template<class CST>
typename CST::size_type dfs_traverse_rec(const CST& cst, const typename CST::node_type &v){
	typename CST::size_type sum_depth = 0;
	typename CST::node_type child = cst.ith_child(v, 1);
	while(child!=cst.root()){
		sum_depth += dfs_traverse_rec(cst, child);
		child = cst.sibling(child);
	}
	if(!cst.is_leaf(v))
		sum_depth += cst.depth(v);
	return sum_depth + cst.depth(v);	
}

template<class CST>
typename CST::size_type child_operation(const CST &cst, const char* text){
	stop_watch sw;
	sw.start();

	typename CST::size_type check = 0, char_pos=0;
	typename CST::node_type v = cst.root();
	for(typename CST::size_type i=0; i<std::min((typename CST::size_type)1000000,cst.csa.size()); ++i)
	{
		v = cst.child(v, text[i], char_pos);
		check += (v!=cst.root());//cst.depth(v);
	}
	sw.stop();
	cout << sw.getUserTime() << " ms /  check = "<< check << endl;

	return check;
}

int main(){
	unsigned char str[12] = "acaaacatat";
	full_compressed_cst_sct cstsct1(str);
	full_compressed_cst_sada cstsada1(str);
	lcp_uncompressed_cst_sct cstsct2(str);
	lcp_uncompressed_cst_sada cstsada2(str);
	full_uncompressed_cst_sct cstsct3(str);
	full_uncompressed_cst_sada cstsada3(str);

	print_tree(cstsct1);
	print_tree(cstsada1);

	lcp_kurtz< full_compressed_cst_sct::csa_type > lcp_k(&cstsct1.csa, str, cstsct1.csa);

	for(uint32_t i=0; i<cstsct1.lcp.size(); ++i){
		cout<<cstsct1.lcp[i]<<" "<<lcp_k[i]<<endl;
	}
	full_compressed_cst_sct_kurtz www(str);
//	return 0;

//	print_tree(cstsct2);
//	print_tree(cstsada2);
//	print_tree(cstsct3);
//	print_tree(cstsada3);


	char *c_str = NULL;
	// Space consumption of the
	// String of size 1 MByte
	if( file::readText("english.50MB", c_str) == 0){
		cout<<"Could not read file \"english.50MB\"! Please download it from the pizza and chili website and place it in the current directory to run the tests"<<endl;
		return 0;
	}
	string bigstr(c_str);
	delete [] c_str;
	full_compressed_cst_sct st1((const unsigned char*)bigstr.c_str());
	st1.print_info();
/*
	full_compressed_cst_sada st2(st1,(const unsigned char*)bigstr.c_str());
	st2.print_info();
	bit_vector::size_type pat=0,skipped=0;
	for(bit_vector::size_type i=0; i<st2.bp.size(); ++i){
		pat = pat<<1;
		pat = pat & 0x7;
		pat += st2.bp[i];
		if(pat==6){
			++skipped;
		}	
	}
	cout<<"bp_size = "<<st2.bp.size()<<" skipped="<<skipped<<endl;


	lcp_uncompressed_cst_sct st3(st1.csa, (const unsigned char*)bigstr.c_str());
	st3.print_info();

	lcp_uncompressed_cst_sada st4(st3, (const unsigned char*)bigstr.c_str());
	st4.print_info();

	full_uncompressed_cst_sct st5((const unsigned char*)bigstr.c_str());
	st5.print_info();

	full_uncompressed_cst_sada st6((const unsigned char*)bigstr.c_str());
	st6.print_info();

	dfs_traverse(st1);
	dfs_traverse(st2);
	dfs_traverse(st3);
	dfs_traverse(st4);
	dfs_traverse(st5);
	dfs_traverse(st6);

	cout<<endl;

	dfs_traverse_recursive(st1);
	dfs_traverse_recursive(st2);
	dfs_traverse_recursive(st3);
	dfs_traverse_recursive(st4);
	dfs_traverse_recursive(st5);
	dfs_traverse_recursive(st6);

	cout<<endl;

	child_operation(st1, bigstr.c_str());
	child_operation(st2, bigstr.c_str());
	child_operation(st3, bigstr.c_str());
	child_operation(st4, bigstr.c_str());
	child_operation(st5, bigstr.c_str());
	child_operation(st6, bigstr.c_str());

	string pattern;
	while(cin>>pattern and pattern.size()>0){
		cout<<algorithm::count(st1, (full_compressed_cst_sct::pattern_type)pattern.c_str(), pattern.size() )<<endl;
		cout<<algorithm::count(st2, (full_compressed_cst_sada::pattern_type)pattern.c_str(), pattern.size() )<<endl;
		cout<<algorithm::count(st2.csa, (full_compressed_cst_sada::csa_type::pattern_type)pattern.c_str(), pattern.size() ) <<endl;
		int_vector<> occ1;
		algorithm::locate(st1, (full_compressed_cst_sct::pattern_type)pattern.c_str(), pattern.size(), occ1 );
		for(size_t i=0; i<occ1.size(); ++i){
			cout<<occ1[i]<<" ";
		}
		cout<<endl;

	}
*/	
	
	return 0;	
}

